#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Unit tests and fixtures for torrentfileQt Application."""


import atexit
import os
import shutil
import string
import sys
from datetime import datetime
from pathlib import Path

import pytest
from torrentfile import TorrentFile, TorrentFileHybrid, TorrentFileV2

from torrentfileQt import alt_start

WINDOW, APP = alt_start()


def exception_hook(exctype, value, traceback):  # pragma:  no cover
    """Except hook capturing."""
    print(exctype, value, traceback)
    sys._excepthook(exctype, value, traceback)


def tempfile(path=None, exp=18):
    """Create temporary file.

    Creates a temporary file for unittesting purposes.py

    Parameters
    ----------
    path : str, optional
        relative path to temporary files, by default None
    exp : int, optional
        Exponent used to determine size of file., by default 18

    Returns
    -------
    str
        absolute path to file.
    """
    seq = (string.printable + string.whitespace).encode("utf-8")
    root = Path(__file__).parent / "TESTDIR"
    if not os.path.exists(root):
        os.mkdir(root)
    if not path:
        path = root / (str(datetime.timestamp(datetime.now())) + ".file")
    parts = Path(path).parts
    partial = root
    for i, part in enumerate(parts):
        partial = partial / part
        if i == len(parts) - 1:
            with open(partial, "wb") as binfile:
                size = 2 ** exp
                while size > 0:
                    if len(seq) < size:
                        binfile.write(seq)
                        size -= len(seq)
                        seq += seq
                    else:
                        binfile.write(seq[:size])
                        size -= size
        else:
            if not os.path.exists(partial):
                os.mkdir(partial)
    return partial


def rmpath(*args):
    """Remove file or directory path.

    Parameters
    ----------
    args : list[str]
        Filesystem locations for removing.
    """
    for arg in args:
        if not os.path.exists(arg):
            continue
        if os.path.isdir(arg):
            try:
                shutil.rmtree(arg)
            except PermissionError:  # pragma: nocover
                pass
        elif os.path.isfile(arg):
            try:
                os.remove(arg)
            except PermissionError:  # pragma: nocover
                pass


def tempdir(ext="1", files=None):
    """Create temporary directory.

    Parameters
    ----------
    ext : str, optional
        extension to file names, by default "1"
    files : `list`
        alternate list of files.

    Returns
    -------
    str
        path to common root for directory.
    """
    if not files:
        files = [
            f"dir{ext}/file1.png",
            f"dir{ext}/file2.mp4",
            f"dir{ext}/file3.mp3",
            f"dir{ext}/file4.zip",
        ]
    paths = []
    for path in files:
        temps = tempfile(path=path, exp=18)
        paths.append(temps)
    return os.path.commonpath(paths)


@atexit.register
def teardown():  # pragma: nocover
    """Remove all temporary directories and files."""
    APP.quit()
    root = Path(__file__).parent / "TESTDIR"
    if os.path.exists(root):
        rmpath(root)


@pytest.fixture(scope="package")
def dir1():
    """Create a specific temporary structured directory.

    Yields
    ------
    str
        path to root of temporary directory
    """
    root = tempdir()
    yield root
    rmpath(root)


@pytest.fixture
def dir2():
    """Create a specific temporary structured directory2.

    Yields
    ------
    str
        path to root of temporary directory
    """
    root = tempdir(ext="2")
    yield root
    rmpath(root)


@pytest.fixture
def dir3():
    """Create a specific temporary structured directory3.

    Yields
    ------
    str
        path to root of temporary directory
    """
    files = [
        "dir3/subdir/file1.jpg",
        "dir3/file2.zip",
        "dir3/subdir/subsubdir/file3.mp3",
        "dir3/file4.txt",
        "dir3/subdir/file5.epub"
    ]
    root = tempdir(files=files)
    yield root
    rmpath(root)


@pytest.fixture(scope="package")
def wind():
    """
    Create a window and application for testing.

    Returns
    -------
    `tuple`
        information to pass to test function.
    """
    window, app = WINDOW, APP
    return window, app


@pytest.fixture(params=[TorrentFile, TorrentFileHybrid, TorrentFileV2])
def ttorrent(request, dir2):
    """
    Generate a metafile for testing.

    Parameters
    ----------
    request : MetaFile
        an instance of torrentfile creator.
    dir2 : `str`
        path to temporary directory

    Yields
    ------
    `str`
        path to new .torrent file.
    """
    args = {
        "path": dir2,
        "outfile": str(dir2) + "t.torrent",
        "comment": "This is a comment",
        "source": "SomeSource",
        "private": 1
    }

    torrent = request.param(**args)
    outfile, _ = torrent.write()
    yield outfile
    rmpath(outfile)



# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for testing procedures on Check Tab."""
import os

import pyben
import pytest

from tests import dir1, dir2, rmpath, tempfile, wind


def test_rmpath():
    """Test the rmpath function."""
    fake = "./ajdiednvikjdod"
    rmpath(fake)
    assert not os.path.exists(fake)


def test_fixtures():
    """Test fixtures."""
    assert dir1 and dir2 and tempfile and wind


def test_create_with_hasher1(dir2, wind):
    """Test the radio buttons on create tab v1 hasher."""
    window, app = wind
    metafile = dir2 + ".torrent"
    creator = window.central.createWidget
    creator.window.central.setCurrentWidget(creator)
    app.processEvents()
    creator.path_input.clear()
    creator.path_input.setText(dir2)
    creator.output_input.clear()
    creator.output_input.setText(metafile)
    creator.v1button.setChecked(True)
    creator.piece_length.setCurrentIndex(2)
    creator.submit_button.click()
    creator.submit_button.join()
    assert os.path.exists(metafile)
    rmpath(metafile)


def test_create_tab_browse(dir2, wind):
    """Test Info tab select1."""
    window, app = wind
    path = dir2
    createtab = window.central.createWidget
    createtab.window.central.setCurrentWidget(createtab)
    app.processEvents()
    button = createtab.browse_file_button
    button.browse(path=path)
    createtab.comment_input.setText("Some Text")
    createtab.source_input.setText("Some Source")
    assert createtab.path_input.text() == path


def test_create_tab_dir(dir2, wind):
    """Test create tab with folder."""
    window, app = wind
    path = dir2
    root = path
    createtab = window.central.createWidget
    button = createtab.browse_dir_button
    window.central.setCurrentWidget(createtab)
    app.processEvents()
    button.browse(path=root)
    torfile = root + ".test.torrent"
    outbutton = createtab.output_button
    outbutton.output(outpath=torfile)
    createtab.announce_input.setPlainText("announce.com")
    createtab.comment_input.setText("comment")
    createtab.private.click()
    submit = createtab.submit_button
    submit.click()
    submit.join()
    assert os.path.exists(torfile)
    rmpath(torfile)


@pytest.mark.parametrize(
    "field",
    [
        "announce",
        "announce-list",
        "source",
        "private",
        "comment",
        "piece length",
    ],
)
def test_create_tab_fields(dir2, field, wind):
    """Test create tab with folder."""
    window, app = wind
    path = dir2
    root = path
    createtab = window.central.createWidget
    button = createtab.browse_dir_button
    button.browse(path=root)
    createtab.window.central.setCurrentWidget(createtab)
    app.processEvents()
    torfile = root + ".test.torrent"
    outbutton = createtab.output_button
    outbutton.output(outpath=torfile)
    createtab.announce_input.setPlainText(
        "https://announce.com\nhttp://announce2.com\nhttp://announce4.com"
    )
    createtab.comment_input.setText("some comment")
    createtab.private.setChecked(True)
    createtab.source_input.setText("TestSource")
    submit = createtab.submit_button
    submit.click()
    submit.join()
    result = pyben.load(torfile)
    assert field in result or field in result["info"]
    rmpath(torfile)


def test_sized_create(wind):
    """Test browse file button on create tab."""
    window, app = wind
    path = str(tempfile(exp=28))
    createtab = window.central.createWidget
    button = createtab.browse_file_button
    button.browse(path=path)
    app.processEvents()
    assert createtab.path_input.text() == path



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for testing procedures on Check Tab."""

import pyben
import pytest

from tests import dir1, dir2, ttorrent, wind


def test_fixtures():
    """Test fixtures."""
    assert dir1 and dir2 and ttorrent and wind


@pytest.mark.parametrize("field", ["announce", "name", "private", "comment"])
def test_editor_torrent_loading(field, wind, ttorrent):
    """Testing editor widget functionality."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    editor.fileButton.browse(ttorrent)
    app.processEvents()
    fields = []
    for i in range(editor.table.rowCount()):
        fields.append(editor.table.item(i, 0).text())
    assert field in fields


def test_editor_torrent_saving(wind, ttorrent):
    """Testing editor widget saving functionality."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    editor.fileButton.browse(ttorrent)
    for i in range(editor.table.rowCount()):
        item1 = editor.table.item(i, 0)
        item2 = editor.table.item(i, 1)
        if item1.text() == "announce":
            item2.setText("other")
            break
    app.processEvents()
    editor.button.click()
    meta = pyben.load(ttorrent)
    assert meta["announce"] == "other"
